﻿using UnityEngine;
using System.Collections;

public class BookObject : MonoBehaviour {

	private const int TURN_NONE = 0;
	private const int TURN_FORWARD = 1;
	private const int TURN_BACK = 2;

	private const int POSITION_UP = 0;
	private const int POSITION_LOWERING = 1;
	private const int POSITION_RAISING = 2;
	private const int POSITION_DOWN = 3;

	public BookCameras bookCameras;
	public int segmentCount;
	public MeshFilter leftPage;
	public MeshFilter rightPage;
	public MeshFilter middlePageLeft;
	public MeshFilter middlePageRight;
	public float pageWidth;
	public float pageHeight;
	public float bendAngle;
	public float turnSpeed;
	public Transform awayPosition;
	public float awaySpeed = 1f;

	private Vector3 [] _newVertices;
	private Vector2 [] _newUV;
	private int [] _newTriangles;
	private float _x;
	private float _z;
	private float _distance;
	private float _ratio;
	private float _radians;
	private float _baseRadians;
	private float _radiansOffset;
	private float _i;

	private float _turn;
	private int _turnMode;

	private float _halfPageHeight;

	private int _firstPage;
	private int _position;

	private MeshRenderer _leftPageRenderer;
	private MeshRenderer _rightPageRenderer;
	private MeshRenderer _middlePageLeftRenderer;
	private MeshRenderer _middlePageRightRenderer;

	private Vector3 _upPosition;
	private Quaternion _upRotation;
	private float _positionRatio;

	// Use this for initialization
	void Start () {
		_halfPageHeight = pageHeight * 0.5f;

		_newVertices = new Vector3 [ ( segmentCount + 1 ) * 2 ];
		_newUV = new Vector2 [ ( segmentCount + 1 ) * 2 ];
		_newTriangles = new int [ segmentCount * 6 ];

		CreateMesh( leftPage.mesh, 0f, true );
		CreateMesh( rightPage.mesh, 1f, false );
		CreateMesh( middlePageLeft.mesh, 0.5f, false );
		CreateMesh( middlePageRight.mesh, 0.5f, true );

		_turn = 0f;
		_turnMode = TURN_NONE;

		_firstPage = 0;

		_leftPageRenderer = leftPage.GetComponent<MeshRenderer>();
		_rightPageRenderer = rightPage.GetComponent<MeshRenderer>();
		_middlePageLeftRenderer = middlePageLeft.GetComponent<MeshRenderer>();
		_middlePageRightRenderer = middlePageRight.GetComponent<MeshRenderer>();

		_middlePageLeftRenderer.enabled = false;
		_middlePageRightRenderer.enabled = false;

		bookCameras.SetFirstPage( _firstPage, false );

		_upPosition = transform.localPosition;
		_upRotation = transform.localRotation;

		_position = POSITION_DOWN;

		transform.localPosition = awayPosition.position;
		transform.localRotation = awayPosition.rotation;

		_positionRatio = 0f;
	}
	
	// Update is called once per frame
	void Update () {

		UpdatePaging();
		UpdatePosition();
	}

	private void UpdatePosition ()
	{
		if ( Input.GetKeyDown( KeyCode.Tab ) )
		{
			if ( _position == POSITION_DOWN )
			{
				_position = POSITION_RAISING;
				_positionRatio = 0f;
			}

			if ( _position == POSITION_UP )
			{
				_position = POSITION_LOWERING;
				_positionRatio = 1f;
			}
		}

		if ( _position == POSITION_RAISING )
		{
			_positionRatio += Time.deltaTime * awaySpeed;
			if ( _positionRatio >= 1f )
			{
				_positionRatio = 1f;
				_position = POSITION_UP;
			}
		}

		if ( _position == POSITION_LOWERING )
		{
			_positionRatio -= Time.deltaTime * awaySpeed;
			if ( _positionRatio <= 0f )
			{
				_positionRatio = 0f;
				_position = POSITION_DOWN;
			}
		}

		transform.localPosition = Vector3.Lerp( awayPosition.localPosition, _upPosition, _positionRatio );
		transform.localRotation = Quaternion.Slerp( awayPosition.localRotation, _upRotation, _positionRatio );
	}

	private void UpdatePaging ()
	{
		if ( _turnMode == TURN_NONE )
		{
			if ( Input.GetKeyDown ( KeyCode.E ) && _firstPage < bookCameras.pageCount - 2 )
			{
				_turnMode = TURN_FORWARD;
				_turn = 1f;
				_middlePageLeftRenderer.enabled = true;
				_middlePageRightRenderer.enabled = true;

				bookCameras.SetFirstPage( _firstPage, true );
			}
			else if ( Input.GetKeyDown( KeyCode.Q ) && _firstPage > 0 )
			{
				_turnMode = TURN_BACK;
				_turn = 0f;
				_middlePageLeftRenderer.enabled = true;
				_middlePageRightRenderer.enabled = true;

				_firstPage -= 2;

				bookCameras.SetFirstPage( _firstPage, true );
			}
		}

		switch ( _turnMode )
		{
			case TURN_BACK:
				_turn+=turnSpeed * Time.deltaTime;
				if ( _turn >= 1f )
				{
					_middlePageLeftRenderer.enabled = false;
					_middlePageRightRenderer.enabled = false;
					_turnMode = TURN_NONE;
					bookCameras.SetFirstPage( _firstPage, false );
				}
				UpdateMesh( middlePageLeft.sharedMesh, _turn, false );
				UpdateMesh( middlePageRight.sharedMesh, _turn, true );
				break;
			case TURN_FORWARD:
				_turn-=turnSpeed * Time.deltaTime;
				if ( _turn <= 0f )
				{
					_middlePageLeftRenderer.enabled = false;
					_middlePageRightRenderer.enabled = false;
					_turnMode = TURN_NONE;
					_firstPage+=2;
					bookCameras.SetFirstPage( _firstPage, false );
				}
				UpdateMesh( middlePageLeft.sharedMesh, _turn, false );
				UpdateMesh( middlePageRight.sharedMesh, _turn, true );
				break;
		}
	}

	private void CreateMesh ( Mesh argMesh, float argPosition, bool argFacing )
	{
		argMesh.Clear();

		for ( int i = 0; i <= segmentCount; i++ )
		{
			_newVertices [ i * 2 + 0 ] = new Vector3();
			_newVertices [ i * 2 + 1 ] = new Vector3();

			_newUV [ i * 2 + 0 ] = new Vector2( (float)i / (float)segmentCount, 0f );
			_newUV [ i * 2 + 1 ] = new Vector2( (float) i / (float) segmentCount, 1f );

			if ( i < segmentCount )
			{
				_newTriangles [ i * 6 + 0 ] = i * 2;
				_newTriangles [ i * 6 + 1 ] = i * 2 + 1;
				_newTriangles [ i * 6 + 2 ] = ( i + 1 ) * 2 + 1;

				_newTriangles [ i * 6 + 3 ] = i * 2;
				_newTriangles [ i * 6 + 4 ] = ( i + 1 ) * 2 + 1;
				_newTriangles [ i * 6 + 5 ] = ( i + 1 ) * 2;
			}
		}

		argMesh.vertices = _newVertices;
		argMesh.uv = _newUV;
		argMesh.triangles = _newTriangles;

		UpdateMesh( argMesh, argPosition, argFacing );
	}

	private void UpdateMesh ( Mesh argMesh, float argPosition, bool argFacing )
	{
		_baseRadians = argPosition * Mathf.PI;

		for ( _i = 0; _i <= segmentCount; _i ++ )
		{
			_ratio = _i / (float) segmentCount;

			if ( argFacing )
			{
				_distance = 1f - _ratio;
			}
			else
			{
				_distance = _ratio;
			}

			_radiansOffset = bendAngle * Mathf.Sin ( _ratio * Mathf.PI ) / ( _distance == 0 ? 1 : _distance );
			_radiansOffset *= Mathf.Lerp( 1, -1, argPosition * argPosition * argPosition );

			_distance = _distance * pageWidth;

			_radians = _baseRadians + _radiansOffset;

			_x = -_distance * Mathf.Cos( _radians );
			_z = -_distance * Mathf.Sin( _radians );

			_newVertices [ (int)_i * 2 + 0 ] = new Vector3( _x, -_halfPageHeight, _z );
			_newVertices [ (int)_i * 2 + 1 ] = new Vector3( _x, _halfPageHeight * 0.5f, _z );
		}

		argMesh.vertices = _newVertices;

		argMesh.RecalculateBounds();
		argMesh.RecalculateNormals();
	}
}
