﻿using UnityEngine;
using System.Collections;

public class PlayerController : MonoBehaviour {

	public Transform verticalAxis;
	public Transform horizontalAxis;
	public float horizontalSensitivity = 100f;
	public float verticalSensitivity = 100f;
	public float forwardSpeed = 3f;
	public float horizontalSpeed = 2f;

	private float _theta;
	private float _phi;
	private Rigidbody _rigidbody;

	// Use this for initialization
	void Start () {
		_phi = 0f;
		_theta = 0f;
		_rigidbody = GetComponent<Rigidbody>();
	}
	
	// Update is called once per frame
	void Update () {
		UpdateRotation ();
	}

	void UpdateRotation ()
	{
		_phi += Input.GetAxis( "Mouse Y" ) * verticalSensitivity * Time.deltaTime;
		_phi = Mathf.Clamp( _phi, -90f, 90f );

		_theta += Input.GetAxis( "Mouse X" ) * horizontalSensitivity * Time.deltaTime;
		
		while ( _theta < 0f )
		{
			_theta += 360f;
		}

		while ( _theta >= 360f )
		{
			_theta -= 360f;
		}

		horizontalAxis.localRotation = Quaternion.AngleAxis(_phi,Vector3.left);
		verticalAxis.localRotation = Quaternion.AngleAxis( _theta, Vector3.up );
	}

	void UpdatePosition ()
	{
		Vector3 newPosition = 
			_rigidbody.position + 
			verticalAxis.forward * Input.GetAxis( "Vertical" ) * Time.deltaTime * forwardSpeed +
			verticalAxis.right * Input.GetAxis ( "Horizontal" ) * Time.deltaTime * horizontalSpeed +
			Vector3.up;

		RaycastHit hitInfo;

		if ( Physics.Raycast ( newPosition, Vector3.down, out hitInfo, 2f, 1 ) )
		{
			newPosition = hitInfo.point + Physics.gravity * Time.deltaTime * 0.00005f;

			_rigidbody.velocity = Vector3.zero;
			_rigidbody.MovePosition( newPosition );
		}
	}

	void FixedUpdate ()
	{
		UpdatePosition ();
	}
}
